@@ -26,7 +26,8 @@
 #include <boost/interprocess/smart_ptr/detail/sp_counted_base.hpp>
 #include <boost/interprocess/smart_ptr/scoped_ptr.hpp>
 #include <boost/interprocess/detail/utilities.hpp>
-#include <boost/pointer_to_other.hpp>
+#include <boost/container/allocator/allocator_traits.hpp>
+#include <boost/intrusive/pointer_traits.hpp>
 
 namespace boost {
 
@@ -62,28 +63,36 @@ struct scoped_ptr_dealloc_functor
    {  if (ptr) priv_deallocate(ptr, alloc_version());  }
 };
 
+   
+
 template<class A, class D>
 class sp_counted_impl_pd 
    :  public sp_counted_base
-   ,  A::template rebind< sp_counted_impl_pd<A, D> >::other
+   ,  boost::container::allocator_traits<A>::template
+         portable_rebind_alloc< sp_counted_impl_pd<A, D> >::type
    ,  D  // copy constructor must not throw
 {
    private:
    typedef sp_counted_impl_pd<A, D>          this_type;
-   typedef typename A::template rebind
-      <this_type>::other                     this_allocator;
-   typedef typename A::template rebind
-      <const this_type>::other               const_this_allocator;
+   typedef typename boost::container::
+      allocator_traits<A>::template
+         portable_rebind_alloc
+            < this_type >::type              this_allocator;
+   typedef typename boost::container::
+      allocator_traits<A>::template
+         portable_rebind_alloc
+            < const this_type >::type        const_this_allocator;
    typedef typename this_allocator::pointer  this_pointer;
 
    sp_counted_impl_pd( sp_counted_impl_pd const & );
    sp_counted_impl_pd & operator= ( sp_counted_impl_pd const & );
 
-   typedef typename boost::pointer_to_other
-            <typename A::pointer, const D>::type   const_deleter_pointer;
-
-   typedef typename boost::pointer_to_other
-            <typename A::pointer, const A>::type   const_allocator_pointer;
+   typedef typename boost::intrusive::
+      pointer_traits<typename A::pointer>::template
+         rebind_pointer<const D>::type                   const_deleter_pointer;
+   typedef typename boost::intrusive::
+      pointer_traits<typename A::pointer>::template
+         rebind_pointer<const A>::type                   const_allocator_pointer;
 
    typedef typename D::pointer   pointer;
    pointer m_ptr;
@@ -115,7 +124,7 @@ class sp_counted_impl_pd
       scoped_ptr< this_type, scoped_ptr_dealloc_functor<this_allocator> >
          deleter(this_ptr, a_copy);
       typedef typename this_allocator::value_type value_type;
-      ipcdetail::get_pointer(this_ptr)->~value_type();
+      ipcdetail::to_raw_pointer(this_ptr)->~value_type();
    }
 
    void release() // nothrow
