@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2005-2009. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2005-2011. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -19,17 +19,14 @@
 #include <boost/interprocess/detail/workaround.hpp>
 #include <boost/interprocess/creation_tags.hpp>
 #include <boost/interprocess/exceptions.hpp>
-#include <boost/interprocess/sync/emulation/named_creation_functor.hpp>
 #include <boost/interprocess/detail/interprocess_tester.hpp>
+#include <boost/interprocess/detail/posix_time_types_wrk.hpp>
 #include <boost/interprocess/permissions.hpp>
 
 #if defined(BOOST_INTERPROCESS_NAMED_MUTEX_USES_POSIX_SEMAPHORES)
-   #include <boost/interprocess/sync/posix/semaphore_wrapper.hpp>
+#include <boost/interprocess/sync/posix/named_mutex.hpp>
 #else
-   #include <boost/interprocess/shared_memory_object.hpp>
-   #include <boost/interprocess/sync/interprocess_mutex.hpp>
-   #include <boost/interprocess/detail/managed_open_or_create_impl.hpp>
-   #include <boost/interprocess/detail/posix_time_types_wrk.hpp>
+#include <boost/interprocess/sync/shm/named_mutex.hpp>
 #endif
 
 //!\file
@@ -108,122 +105,53 @@ class named_mutex
    void dont_close_on_destruction();
 
    #if defined(BOOST_INTERPROCESS_NAMED_MUTEX_USES_POSIX_SEMAPHORES)
-   ipcdetail::named_semaphore_wrapper m_sem;
+   typedef ipcdetail::posix_named_mutex   impl_t;
+   impl_t m_mut;
    #else
+   typedef ipcdetail::shm_named_mutex     impl_t;
+   impl_t m_mut;
+   public:
    interprocess_mutex *mutex() const
-   {  return static_cast<interprocess_mutex*>(m_shmem.get_user_address()); }
-
-   ipcdetail::managed_open_or_create_impl<shared_memory_object> m_shmem;
-   typedef ipcdetail::named_creation_functor<interprocess_mutex> construct_func_t;
+   {  return m_mut.mutex(); }
    #endif
+
    /// @endcond
 };
 
 /// @cond
 
-#if defined(BOOST_INTERPROCESS_NAMED_MUTEX_USES_POSIX_SEMAPHORES)
-
 inline named_mutex::named_mutex(create_only_t, const char *name, const permissions &perm)
-   :  m_sem(ipcdetail::DoCreate, name, 1, perm)
+   :  m_mut(create_only_t(), name, perm)
 {}
 
 inline named_mutex::named_mutex(open_or_create_t, const char *name, const permissions &perm)
-   :  m_sem(ipcdetail::DoOpenOrCreate, name, 1, perm)
+   :  m_mut(open_or_create_t(), name, perm)
 {}
 
 inline named_mutex::named_mutex(open_only_t, const char *name)
-   :  m_sem(ipcdetail::DoOpen, name, 1, permissions())
+   :  m_mut(open_only_t(), name)
 {}
 
 inline void named_mutex::dont_close_on_destruction()
-{  ipcdetail::interprocess_tester::dont_close_on_destruction(m_sem);  }
+{  ipcdetail::interprocess_tester::dont_close_on_destruction(m_mut); }
 
 inline named_mutex::~named_mutex()
 {}
 
 inline void named_mutex::lock()
-{  m_sem.wait();  }
+{  m_mut.lock();  }
 
 inline void named_mutex::unlock()
-{  m_sem.post();  }
+{  m_mut.unlock();  }
 
 inline bool named_mutex::try_lock()
-{  return m_sem.try_wait();  }
+{  return m_mut.try_lock();  }
 
 inline bool named_mutex::timed_lock(const boost::posix_time::ptime &abs_time)
-{
-   if(abs_time == boost::posix_time::pos_infin){
-      this->lock();
-      return true;
-   }
-   return m_sem.timed_wait(abs_time);
-}
+{  return m_mut.timed_lock(abs_time);  }
 
 inline bool named_mutex::remove(const char *name)
-{  return ipcdetail::named_semaphore_wrapper::remove(name);   }
-
-#else
-
-inline void named_mutex::dont_close_on_destruction()
-{  ipcdetail::interprocess_tester::dont_close_on_destruction(m_shmem);  }
-
-inline named_mutex::~named_mutex()
-{}
-
-inline named_mutex::named_mutex(create_only_t, const char *name, const permissions &perm)
-   :  m_shmem  (create_only
-               ,name
-               ,sizeof(interprocess_mutex) +
-                  ipcdetail::managed_open_or_create_impl<shared_memory_object>::
-                     ManagedOpenOrCreateUserOffset
-               ,read_write
-               ,0
-               ,construct_func_t(ipcdetail::DoCreate)
-               ,perm)
-{}
-
-inline named_mutex::named_mutex(open_or_create_t, const char *name, const permissions &perm)
-   :  m_shmem  (open_or_create
-               ,name
-               ,sizeof(interprocess_mutex) +
-                  ipcdetail::managed_open_or_create_impl<shared_memory_object>::
-                     ManagedOpenOrCreateUserOffset
-               ,read_write
-               ,0
-               ,construct_func_t(ipcdetail::DoOpenOrCreate)
-               ,perm)
-{}
-
-inline named_mutex::named_mutex(open_only_t, const char *name)
-   :  m_shmem  (open_only
-               ,name
-               ,read_write
-               ,0
-               ,construct_func_t(ipcdetail::DoOpen))
-{}
-
-inline void named_mutex::lock()
-{  this->mutex()->lock();  }
-
-inline void named_mutex::unlock()
-{  this->mutex()->unlock();  }
-
-inline bool named_mutex::try_lock()
-{  return this->mutex()->try_lock();  }
-
-inline bool named_mutex::timed_lock(const boost::posix_time::ptime &abs_time)
-{
-   if(abs_time == boost::posix_time::pos_infin){
-      this->lock();
-      return true;
-   }
-   return this->mutex()->timed_lock(abs_time);
-}
-
-inline bool named_mutex::remove(const char *name)
-{  return shared_memory_object::remove(name); }
-
-#endif
+{  return impl_t::remove(name);   }
 
 /// @endcond
 
