@@ -1,5 +1,5 @@
 /*-----------------------------------------------------------------------------+    
-Copyright (c) 2008-2009: Joachim Faulhaber
+Copyright (c) 2008-2012: Joachim Faulhaber
 +------------------------------------------------------------------------------+
    Distributed under the Boost Software License, Version 1.0.
       (See accompanying file LICENCE.txt or copy at
@@ -1516,5 +1516,36 @@ void interval_map_intersects_4_bicremental_types()
 }
 
 
+template 
+<
+#if (defined(__GNUC__) && (__GNUC__ < 4)) //MEMO Can be simplified, if gcc-3.4 is obsolete
+    ICL_IntervalMap_TEMPLATE(T,U,Traits,partial_absorber) IntervalMap,
+#else
+    ICL_IntervalMap_TEMPLATE(_T,_U,Traits,partial_absorber) IntervalMap,
+#endif
+    class T, class U
+>
+void interval_map_move_4_discrete_types()
+{
+#   ifndef BOOST_NO_RVALUE_REFERENCES
+    typedef IntervalMap<T,U> IntervalMapT;
+    typedef typename IntervalMapT::interval_type   IntervalT;
+
+    IntervalMapT map_A(boost::move(IntervalMapT(IDv(0,4,2))));
+    IntervalMapT map_B(boost::move(IntervalMapT(IDv(0,2,1)).add(IDv(2,4,1)).add(IDv(0,4,1))));
+
+    BOOST_CHECK( icl::is_element_equal(map_A, map_B) );
+    BOOST_CHECK_EQUAL( map_A, join(map_B) );
+
+    map_A = boost::move(IntervalMapT(IIv(1,4,2)));
+    map_B = boost::move(IntervalMapT(CIv(0,2,1)).insert(IDv(3,5,1)).add(CDv(0,5,1)));
+
+    BOOST_CHECK( icl::is_element_equal(map_A, map_B) );
+    BOOST_CHECK_EQUAL( map_A, join(map_B) );
+
+#   endif // BOOST_NO_RVALUE_REFERENCES
+}
+
+
 #endif // LIBS_ICL_TEST_TEST_INTERVAL_MAP_SHARED_HPP_JOFA_081005
 
