@@ -2,18 +2,23 @@
     Boost.Wave: A Standard compliant C++ preprocessor library
     http://www.boost.org/
 
-    Copyright (c) 2001-2009 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2011 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
 
+// disable stupid compiler warnings
+#include <boost/config/warning_disable.hpp>
+
 // system headers
 #include <string>
 #include <iostream>
 #include <vector>
+#include <ctime>
 
 // include boost
 #include <boost/config.hpp>
+#include <boost/assert.hpp>
 #include <boost/throw_exception.hpp>
 #include <boost/filesystem/path.hpp>
 #include <boost/filesystem/operations.hpp>
@@ -35,13 +40,21 @@
 #include "testwave_app.hpp"
 #include "collect_hooks_information.hpp"
 
+# ifdef BOOST_NO_STDC_NAMESPACE
+namespace std 
+{ 
+    using ::asctime; using ::gmtime; using ::localtime;
+    using ::difftime; using ::time; using ::tm; using ::mktime; using ::system; 
+}
+# endif
+
 namespace po = boost::program_options;
 namespace fs = boost::filesystem;
 
 ///////////////////////////////////////////////////////////////////////////////
 // testwave version definitions
 #define TESTWAVE_VERSION_MAJOR           0
-#define TESTWAVE_VERSION_MINOR           5
+#define TESTWAVE_VERSION_MINOR           6
 #define TESTWAVE_VERSION_SUBMINOR        0
 
 namespace {
@@ -53,8 +66,8 @@ namespace {
         std::string &result)
     {
         typedef typename Iterator::value_type token_type;
-        
-        token_type tok = *it++;    
+
+        token_type tok = *it++;
         result = result + tok.get_value().c_str();
         return (it == end) ? false : true;
     }
@@ -64,11 +77,11 @@ namespace {
     String const& handle_quoted_filepath(String &name)
     {
         using boost::wave::util::impl::unescape_lit;
-        
-        String unesc_name = unescape_lit(name.substr(1, name.size()-2));
-        fs::path p (unesc_name.c_str(), fs::native);
 
-        name = String("\"") + p.leaf().c_str() + String("\"");
+        String unesc_name = unescape_lit(name.substr(1, name.size()-2));
+        fs::path p (boost::wave::util::create_path(unesc_name.c_str()));
+
+        name = String("\"") + boost::wave::util::leaf(p).c_str() + String("\"");
         return name;
     }
 
@@ -79,7 +92,7 @@ namespace {
     {
         typedef typename Iterator::value_type token_type;
         typedef typename token_type::string_type string_type;
-        
+
         if (!handle_next_token(it, end, result) ||  // #line
             !handle_next_token(it, end, result) ||  // whitespace
             !handle_next_token(it, end, result) ||  // number
@@ -87,7 +100,7 @@ namespace {
         {
             return false;
         }
-        
+
         using boost::wave::util::impl::unescape_lit;
         
         token_type filename = *it;
@@ -133,11 +146,11 @@ testwave_app::got_expected_result(std::string const& filename,
     std::string const& result, std::string& expected)
 {
     using boost::wave::util::impl::escape_lit;
-    
+
     std::string full_result;
     std::string::size_type pos = 0;
     std::string::size_type pos1 = expected.find_first_of("$");
-    
+
     if (pos1 != std::string::npos) {
         do {
             switch(expected[pos1+1]) {
@@ -168,7 +181,7 @@ testwave_app::got_expected_result(std::string const& filename,
                     }
                 }
                 break;
-                
+
             case 'F':       // insert base file name
                 full_result = full_result + 
                     expected.substr(pos, pos1-pos) + escape_lit(filename);
@@ -176,13 +189,14 @@ testwave_app::got_expected_result(std::string const& filename,
                 break;
 
             case 'P':       // insert full path
-            case 'B':       // same as 'P', but forward slashs on Windows
+            case 'B':       // same as 'P', but forward slashes on Windows
                 {
                     fs::path fullpath (
-                        fs::complete(
-                            fs::path(filename, fs::native), fs::current_path())
+                        boost::wave::util::complete_path(
+                            boost::wave::util::create_path(filename), 
+                            boost::wave::util::current_path())
                         );
-                        
+
                     if ('(' == expected[pos1+2]) {
                     // the $P(basename) syntax is used
                         std::size_t p = expected.find_first_of(")", pos1+1);
@@ -193,16 +207,31 @@ testwave_app::got_expected_result(std::string const& filename,
                             return false;
                         }
                         std::string base = expected.substr(pos1+3, p-pos1-3);
-                        fullpath = fullpath.branch_path() / 
-                            fs::path(base, fs::native);
+                        fullpath = boost::wave::util::branch_path(fullpath) / 
+                            boost::wave::util::create_path(base);
                         full_result += expected.substr(pos, pos1-pos);
                         if ('P' == expected[pos1+1]) {
-                            full_result += 
-                                escape_lit(fullpath.normalize().native_file_string());
+#if defined(BOOST_WINDOWS)
+                            std::string p = replace_slashes(
+                                boost::wave::util::native_file_string(
+                                    boost::wave::util::normalize(fullpath)),
+                                "/", '\\');
+#else
+                            std::string p (
+                                boost::wave::util::native_file_string(
+                                    boost::wave::util::normalize(fullpath)));
+#endif
+                            full_result += escape_lit(p);
                         }
                         else {
-                            full_result += 
-                                escape_lit(fullpath.normalize().string());
+#if defined(BOOST_WINDOWS)
+                            std::string p = replace_slashes(
+                                boost::wave::util::normalize(fullpath).string());
+#else
+                            std::string p (
+                                boost::wave::util::normalize(fullpath).string());
+#endif
+                            full_result += escape_lit(p);
                         }
                         pos1 = expected.find_first_of ("$", 
                             pos = pos1 + 4 + base.size());
@@ -211,18 +240,82 @@ testwave_app::got_expected_result(std::string const& filename,
                     // the $P is used on its own
                         full_result += expected.substr(pos, pos1-pos);
                         if ('P' == expected[pos1+1]) {
-                            full_result += 
-                                escape_lit(fullpath.native_file_string());
+                            full_result += escape_lit(
+                                boost::wave::util::native_file_string(fullpath));
                         }
                         else {
-                            full_result += 
-                                escape_lit(fullpath.string());
+#if defined(BOOST_WINDOWS)
+                            std::string p = replace_slashes(fullpath.string());
+#else
+                            std::string p (fullpath.string());
+#endif
+                            full_result += escape_lit(fullpath.string());
                         }
                         pos1 = expected.find_first_of ("$", pos = pos1 + 2);
                     }
                 }
                 break;
-                
+
+            case 'R':       // insert relative file name
+            case 'S':       // same as 'R', but forward slashes on Windows 
+                {
+                    fs::path relpath;
+                    boost::wave::util::as_relative_to(
+                        boost::wave::util::create_path(filename), 
+                        boost::wave::util::current_path(),
+                        relpath);
+
+                    if ('(' == expected[pos1+2]) {
+                    // the $R(basename) syntax is used
+                        std::size_t p = expected.find_first_of(")", pos1+1);
+                        if (std::string::npos == p) {
+                            std::cerr 
+                                << "testwave: unmatched parenthesis in $R"
+                                    " directive" << std::endl;
+                            return false;
+                        }
+                        std::string base = expected.substr(pos1+3, p-pos1-3);
+                        relpath = boost::wave::util::branch_path(relpath) / 
+                            boost::wave::util::create_path(base);
+                        full_result += expected.substr(pos, pos1-pos);
+                        if ('R' == expected[pos1+1]) {
+                            full_result += escape_lit(
+                                boost::wave::util::native_file_string(
+                                    boost::wave::util::normalize(relpath)));
+                        }
+                        else {
+#if defined(BOOST_WINDOWS)
+                            std::string p = replace_slashes(
+                                boost::wave::util::normalize(relpath).string());
+#else
+                            std::string p (
+                                boost::wave::util::normalize(relpath).string());
+#endif
+                            full_result += escape_lit(p);
+                        }
+                        pos1 = expected.find_first_of ("$", 
+                            pos = pos1 + 4 + base.size());
+                    }
+                    else {
+                    // the $R is used on its own
+                        full_result += expected.substr(pos, pos1-pos);
+                        if ('R' == expected[pos1+1]) {
+                            full_result += escape_lit(
+                                boost::wave::util::native_file_string(relpath));
+                        }
+                        else {
+#if defined(BOOST_WINDOWS)
+                            std::string p = replace_slashes(relpath.string());
+#else
+                            std::string p (relpath.string());
+#endif
+                            full_result += escape_lit(p);
+                        }
+                        pos1 = expected.find_first_of ("$", pos = pos1 + 2);
+                    }
+                }
+                break;
+
             case 'V':       // insert Boost version
                 full_result = full_result + 
                     expected.substr(pos, pos1-pos) + BOOST_LIB_VERSION;
@@ -242,7 +335,7 @@ testwave_app::got_expected_result(std::string const& filename,
     else {
         full_result = expected;
     }
-    
+
     expected = full_result;
     return full_result == result;
 }
@@ -273,6 +366,10 @@ testwave_app::testwave_app(po::variables_map const& vm)
 #endif 
 #if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
         ("noguard,G", "disable include guard detection")
+#endif
+        ("skipped_token_hooks", "record skipped_token hook calls")
+#if BOOST_WAVE_SUPPORT_CPP0X != 0
+        ("c++0x", "enable C99 mode (implies --variadics and --long_long)")
 #endif
     ;
 }
@@ -494,7 +591,7 @@ testwave_app::print_copyright()
         "Testwave: A test driver for the Boost.Wave C++ preprocessor library",
         "http://www.boost.org/",
         "",
-        "Copyright (c) 2001-2009 Hartmut Kaiser, Distributed under the Boost",
+        "Copyright (c) 2001-2011 Hartmut Kaiser, Distributed under the Boost",
         "Software License, Version 1.0. (See accompanying file",
         "LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)",
         0
@@ -556,7 +653,7 @@ namespace {
             value.clear();
         else {
             std::string::size_type last = value.find_last_not_of(" \t");
-            assert(std::string::npos != last);
+            BOOST_ASSERT(std::string::npos != last);
             value = value.substr(first, last-first+1);
         }
         return value;
@@ -778,6 +875,13 @@ testwave_app::initialise_options(Context& ctx, po::variables_map const& vm,
         std::cerr << "initialise_options: initializing options" << std::endl;
     }
 
+    if (vm.count("skipped_token_hooks")) {
+        if (9 == debuglevel) {
+            std::cerr << "initialise_options: option: skipped_token_hooks" << std::endl;
+        }
+        ctx.get_hooks().set_skipped_token_hooks(true);
+    }
+
 //  initialize the given context from the parsed options
 #if BOOST_WAVE_SUPPORT_VARIADICS_PLACEMARKERS != 0
 // enable C99 mode, if appropriate (implies variadics)
@@ -807,6 +911,28 @@ testwave_app::initialise_options(Context& ctx, po::variables_map const& vm,
     }
 #endif // BOOST_WAVE_SUPPORT_VARIADICS_PLACEMARKERS != 0
 
+#if BOOST_WAVE_SUPPORT_CPP0X
+    if (vm.count("c++0x")) {
+        if (9 == debuglevel) {
+            std::cerr << "initialise_options: option: c++0x" << std::endl;
+        }
+        ctx.set_language(
+            boost::wave::language_support(
+                boost::wave::support_cpp0x
+              |  boost::wave::support_option_convert_trigraphs 
+              |  boost::wave::support_option_long_long 
+              |  boost::wave::support_option_emit_line_directives 
+#if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
+              |  boost::wave::support_option_include_guard_detection
+#endif
+#if BOOST_WAVE_EMIT_PRAGMA_DIRECTIVES != 0
+              |  boost::wave::support_option_emit_pragma_directives
+#endif
+              |  boost::wave::support_option_insert_whitespace
+            ));
+    }
+#endif
+
 // enable long_long mode, if appropriate
     if (vm.count("long_long")) {
         if (9 == debuglevel) {
@@ -845,6 +971,7 @@ testwave_app::initialise_options(Context& ctx, po::variables_map const& vm,
             std::cerr << "initialise_options: option: single_line" << std::endl;
         }
         ctx.set_language(boost::wave::enable_single_line(ctx.get_language()));
+        ctx.set_language(boost::wave::enable_emit_line_directives(ctx.get_language(), false));
     }
     
 //  add include directories to the system include search paths
@@ -945,7 +1072,7 @@ testwave_app::initialise_options(Context& ctx, po::variables_map const& vm,
                 std::cerr << "initialise_options: option: -U" << *cit 
                           << std::endl;
             }
-            ctx.remove_macro_definition((*cit).c_str());
+            ctx.remove_macro_definition(*cit);
         }
     }
 
