@@ -16,10 +16,9 @@ namespace quickbook
    quickbook_grammar::quickbook_grammar(quickbook::actions& a)
         : impl_(new impl(a))
         , command_line_macro(impl_->command_line, "command_line_macro")
-        , common(impl_->common, "phrase")
-        , simple_phrase(impl_->simple_phrase, "simple_phrase")
+        , inline_phrase(impl_->inline_phrase, "inline_phrase")
+        , phrase(impl_->phrase_start, "phrase")
         , block(impl_->block_start, "block")
-        , block_skip_initial_spaces(impl_->block_skip_initial_spaces, "block")
         , doc_info(impl_->doc_info_details, "doc_info")
     {
     }
@@ -30,7 +29,7 @@ namespace quickbook
 
     quickbook_grammar::impl::impl(quickbook::actions& a)
         : actions(a)
-        , store_()
+        , cleanup_()
     {
         init_main();
         init_block_elements();
